@@ -38,7 +38,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#if defined(__linux__)
+#if defined(__linux__) || defined(__GNU__)
 #include <sys/vfs.h>
 #elif defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__FreeBSD_kernel__)
 #include <sys/param.h>
@@ -47,10 +47,8 @@
 
 #include <gtk/gtk.h>
 #include <libxfce4util/libxfce4util.h>
-#include <libxfcegui4/libxfcegui4.h>
-#include <libxfce4panel/xfce-panel-plugin.h>
-#include <libxfce4panel/xfce-panel-convenience.h>
-#include <libxfce4panel/xfce-hvbox.h>
+#include <libxfce4ui/libxfce4ui.h>
+#include <libxfce4panel/libxfce4panel.h>
 
 #define ICON_NORMAL             0
 #define ICON_WARNING            1
@@ -63,6 +61,12 @@
 #define COLOR_WARNING           "#FFE500"
 #define COLOR_URGENT            "#FF4F00"
 
+#ifdef LIBXFCE4PANEL_CHECK_VERSION
+#if LIBXFCE4PANEL_CHECK_VERSION (4,9,0)
+#define HAS_PANEL_49
+#endif
+#endif
+
 // }}}
 
 // struct {{{
@@ -141,6 +145,10 @@ fsguard_set_icon (FsGuard *fsguard, gint id)
 
     fsguard->icon_id = id;
 	size = xfce_panel_plugin_get_size (fsguard->plugin);
+#ifdef HAS_PANEL_49
+    size /= xfce_panel_plugin_get_nrows (fsguard->plugin);
+#endif
+
     size -= 2 + 2 * MAX (fsguard->btn_panel->style->xthickness,
                          fsguard->btn_panel->style->ythickness);
 
@@ -218,7 +226,7 @@ __open_mnt (gchar *command, gchar *path)
     gchar *path_quoted;
     path_quoted = g_shell_quote (path);
     cmd = g_strdup_printf ("%s %s", command, path_quoted);
-    res = xfce_exec (cmd, FALSE, FALSE, NULL);
+    res = xfce_spawn_command_line_on_screen (NULL, cmd, FALSE, FALSE, NULL);
     g_free (path_quoted);
     g_free (cmd);
     return res;
@@ -239,16 +247,15 @@ fsguard_open_mnt (GtkWidget *widget, FsGuard *fsguard)
 
     GtkWidget *dialog;
     dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_NO_SEPARATOR, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
-                                     "Free Space Checker", NULL);
+                                     "Free Space Checker");
     gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                              _("Unable to find an appropriate application to open the mount point"),
-                                              NULL);
+                                              _("Unable to find an appropriate application to open the mount point"));
 }
 
 static gboolean
 fsguard_check_fs (FsGuard *fsguard)
 {
-    float               free = 0;
+    float               freespace = 0;
     float               total = 0;
     float               freeblocks = 0;
     float               totalblocks = 0;
@@ -264,12 +271,12 @@ fsguard_check_fs (FsGuard *fsguard)
         blocksize       = fsd.f_bsize;
         freeblocks      = fsd.f_bavail;
         totalblocks     = fsd.f_blocks;
-        free            = (freeblocks * blocksize) / 1048576;
+        freespace       = (freeblocks * blocksize) / 1048576;
         total           = (totalblocks * blocksize) / 1048576;
 
-        if (free > (total * fsguard->limit_warning / 100)) {
+        if (freespace > (total * fsguard->limit_warning / 100)) {
             icon_id = ICON_NORMAL;
-        } else if (free > (total * fsguard->limit_urgent / 100) && free <= (total * fsguard->limit_warning / 100)) {
+        } else if (freespace > (total * fsguard->limit_urgent / 100) && freespace <= (total * fsguard->limit_warning / 100)) {
             icon_id = ICON_WARNING;
         } else {
             icon_id = ICON_URGENT;
@@ -279,14 +286,14 @@ fsguard_check_fs (FsGuard *fsguard)
     /* msg_total_size, msg_size */
     if (total > 1024) {
         g_snprintf (msg_total_size, sizeof(msg_total_size), _("%.2f GB"), total / 1024);
-        g_snprintf (msg_size, sizeof (msg_size), _("%.2f GB"), free / 1024);
+        g_snprintf (msg_size, sizeof (msg_size), _("%.2f GB"), freespace / 1024);
         g_snprintf (msg, sizeof (msg),
                     (*(fsguard->name) != '\0' && strcmp(fsguard->path, fsguard->name)) ?
                     _("%s/%s space left on %s (%s)") : _("%s/%s space left on %s"),
                     msg_size, msg_total_size, fsguard->path, fsguard->name);
     } else {
         g_snprintf (msg_total_size, sizeof (msg_total_size), _("%.0f MB"), total);
-        g_snprintf (msg_size, sizeof (msg_size), _("%.0f MB"), free);
+        g_snprintf (msg_size, sizeof (msg_size), _("%.0f MB"), freespace);
         g_snprintf (msg, sizeof (msg),
                     _("could not check mountpoint %s, please check your config"),
                     fsguard->path);
@@ -298,7 +305,7 @@ fsguard_check_fs (FsGuard *fsguard)
     }
     if (fsguard->show_progress_bar) {
         gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR(fsguard->progress_bar),
-                                       (total > 0 ) ? 1.0 - (free / total) : 0.0);
+                                       (total > 0 ) ? 1.0 - (freespace / total) : 0.0);
         fsguard_refresh_monitor (fsguard);
     }
 
@@ -308,10 +315,10 @@ fsguard_check_fs (FsGuard *fsguard)
     if (err != -1 && !fsguard->seen && icon_id == ICON_URGENT) {
         fsguard->seen = TRUE;
         if (*(fsguard->name) != '\0' && strcmp(fsguard->path, fsguard->name)) {
-            xfce_warn (_("Only %s space left on %s (%s)!"),
+            xfce_dialog_show_warning (NULL, NULL, _("Only %s space left on %s (%s)!"),
                        msg_size, fsguard->path, fsguard->name);
         } else {
-            xfce_warn (_("Only %s space left on %s!"),
+            xfce_dialog_show_warning (NULL, NULL, _("Only %s space left on %s!"),
                        msg_size, fsguard->path);
         }
     }
@@ -325,15 +332,29 @@ fsguard_read_config (FsGuard *fsguard)
     char               *file;
     XfceRc             *rc;
 
-    file = xfce_panel_plugin_save_location (fsguard->plugin, TRUE);
+    /* prepare default values */
+    fsguard->seen               = FALSE;
+    fsguard->name               = g_strdup ("");
+    fsguard->show_name          = FALSE;
+    fsguard->path               = g_strdup ("/");
+    fsguard->show_size          = TRUE;
+    fsguard->show_progress_bar  = TRUE;
+    fsguard->hide_button        = FALSE;
+    fsguard->limit_warning      = 8;
+    fsguard->limit_urgent       = 2;
+
+    file = xfce_panel_plugin_lookup_rc_file(fsguard->plugin);
+    if (!file)
+        return;
     DBG ("Lookup rc file `%s'", file);
-    rc = xfce_rc_simple_open (file, FALSE);
+    rc = xfce_rc_simple_open (file, TRUE);
     g_free (file);
     g_return_if_fail (rc);
 
-    fsguard->seen               = FALSE;
+    g_free (fsguard->name);
     fsguard->name               = g_strdup (xfce_rc_read_entry (rc, "label", ""));
     fsguard->show_name          = xfce_rc_read_bool_entry (rc, "label_visible", FALSE);
+    g_free (fsguard->path);
     fsguard->path               = g_strdup (xfce_rc_read_entry (rc, "mnt", "/"));
     fsguard->show_size          = xfce_rc_read_bool_entry (rc, "lab_size_visible", TRUE);
     fsguard->show_progress_bar  = xfce_rc_read_bool_entry (rc, "progress_bar_visible", TRUE);
@@ -376,6 +397,8 @@ static FsGuard *
 fsguard_new (XfcePanelPlugin *plugin)
 {
     FsGuard *fsguard = g_new0(FsGuard, 1);
+    GtkWidget *alignment;
+
     fsguard->plugin = plugin;
 
     fsguard_read_config (fsguard);
@@ -383,16 +406,16 @@ fsguard_new (XfcePanelPlugin *plugin)
     tooltips = gtk_tooltips_new ();
 
     fsguard->ebox = gtk_event_box_new();
+    gtk_event_box_set_visible_window(GTK_EVENT_BOX(fsguard->ebox), FALSE);
 
     GtkOrientation orientation = xfce_panel_plugin_get_orientation (plugin);
-    fsguard->box =
-      xfce_hvbox_new (orientation == GTK_ORIENTATION_HORIZONTAL ?
-                      GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL,
-                      FALSE, 2);
+    fsguard->box = xfce_hvbox_new (orientation, FALSE, 2);
 
     fsguard->lab_name = gtk_label_new (NULL);
     fsguard->lab_size = gtk_label_new (NULL);
-    fsguard->lab_box = gtk_vbox_new (FALSE, 0);
+    fsguard->lab_box = xfce_hvbox_new (GTK_ORIENTATION_VERTICAL, FALSE, 2);
+
+    alignment = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
 
     fsguard->btn_panel = xfce_create_panel_button ();
     fsguard->icon_panel = gtk_image_new ();
@@ -402,11 +425,7 @@ fsguard_new (XfcePanelPlugin *plugin)
     gtk_progress_bar_set_orientation (GTK_PROGRESS_BAR(fsguard->progress_bar),
                                       orientation == GTK_ORIENTATION_HORIZONTAL ?
                                       GTK_PROGRESS_BOTTOM_TO_TOP : GTK_PROGRESS_LEFT_TO_RIGHT);
-    fsguard->pb_box =
-      xfce_hvbox_new (orientation == GTK_ORIENTATION_HORIZONTAL ?
-                      GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL,
-                      FALSE, 0);
-    gtk_container_set_border_width (GTK_CONTAINER (fsguard->pb_box), BORDER / 2);
+    fsguard->pb_box = xfce_hvbox_new (orientation, FALSE, 0);
 
     g_signal_connect (G_OBJECT(fsguard->btn_panel),
                       "clicked",
@@ -418,7 +437,8 @@ fsguard_new (XfcePanelPlugin *plugin)
     gtk_container_add (GTK_CONTAINER(fsguard->btn_panel), fsguard->icon_panel);
     gtk_container_add (GTK_CONTAINER(fsguard->lab_box), fsguard->lab_name);
     gtk_container_add (GTK_CONTAINER(fsguard->lab_box), fsguard->lab_size);
-    gtk_container_add (GTK_CONTAINER(fsguard->box), fsguard->lab_box);
+    gtk_container_add (GTK_CONTAINER(fsguard->box), alignment);
+    gtk_container_add (GTK_CONTAINER(alignment), fsguard->lab_box);
     gtk_container_add (GTK_CONTAINER(fsguard->box), fsguard->pb_box);
     gtk_container_add (GTK_CONTAINER(fsguard->pb_box), fsguard->progress_bar);
 
@@ -452,6 +472,68 @@ fsguard_free (XfcePanelPlugin *plugin, FsGuard *fsguard)
     g_free(fsguard);
 }
 
+static gboolean
+fsguard_set_size (XfcePanelPlugin *plugin, int size, FsGuard *fsguard)
+{
+    int border_width = (size > 26 ? 2 : 1);
+#ifdef HAS_PANEL_49
+    size /= xfce_panel_plugin_get_nrows (plugin);
+#endif
+    DBG ("Set size to `%d'", size);
+
+    gtk_container_set_border_width (GTK_CONTAINER (fsguard->pb_box), border_width);
+
+    GtkOrientation orientation = xfce_panel_plugin_get_orientation (plugin);
+    if (orientation == GTK_ORIENTATION_HORIZONTAL) {
+        gtk_widget_set_size_request (GTK_WIDGET(fsguard->progress_bar), 8, -1);
+        gtk_widget_set_size_request (GTK_WIDGET(plugin), -1, size);
+    } else {
+        gtk_widget_set_size_request (GTK_WIDGET(fsguard->progress_bar), -1, 8);
+        gtk_widget_set_size_request (GTK_WIDGET(plugin), size, -1);
+    }
+    gtk_widget_set_size_request (fsguard->btn_panel, size, size);
+    size -= 2 * border_width;
+    gtk_widget_set_size_request (fsguard->icon_panel, size, size);
+
+    fsguard_refresh_icon (fsguard);
+
+    return TRUE;
+}
+
+#ifdef HAS_PANEL_49
+static void
+fsguard_set_mode (XfcePanelPlugin *plugin, XfcePanelPluginMode mode, FsGuard *fsguard)
+{
+    GtkOrientation orientation, panel_orientation;
+
+    orientation =
+      (mode != XFCE_PANEL_PLUGIN_MODE_VERTICAL) ?
+      GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL;
+    panel_orientation =
+      (mode == XFCE_PANEL_PLUGIN_MODE_HORIZONTAL) ?
+      GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL;
+
+    DBG ("Set mode to `%s'", mode == XFCE_PANEL_PLUGIN_MODE_HORIZONTAL ?
+         "Horizontal" : (mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL ? "Vertical" : "Deskbar"));
+
+    xfce_hvbox_set_orientation (XFCE_HVBOX (fsguard->box), panel_orientation);
+    xfce_hvbox_set_orientation (XFCE_HVBOX (fsguard->pb_box), panel_orientation);
+    gtk_progress_bar_set_orientation (GTK_PROGRESS_BAR(fsguard->progress_bar),
+                                      panel_orientation == GTK_ORIENTATION_HORIZONTAL ?
+                                      GTK_PROGRESS_BOTTOM_TO_TOP : GTK_PROGRESS_LEFT_TO_RIGHT);
+    gtk_label_set_angle (GTK_LABEL(fsguard->lab_name),
+                         orientation == GTK_ORIENTATION_VERTICAL ? -90 : 0);
+    gtk_label_set_angle (GTK_LABEL(fsguard->lab_size),
+                         orientation == GTK_ORIENTATION_VERTICAL ? -90 : 0);
+    xfce_hvbox_set_orientation (XFCE_HVBOX (fsguard->lab_box),
+                                orientation == GTK_ORIENTATION_VERTICAL ?
+                                GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL);
+    gtk_box_reorder_child (GTK_BOX (fsguard->lab_box),
+                           orientation == GTK_ORIENTATION_VERTICAL ? fsguard->lab_size : fsguard->lab_name, 0);
+    fsguard_set_size (plugin, xfce_panel_plugin_get_size (plugin), fsguard);
+}
+
+#else
 static void
 fsguard_set_orientation (XfcePanelPlugin *plugin, GtkOrientation orientation, FsGuard *fsguard)
 {
@@ -463,28 +545,9 @@ fsguard_set_orientation (XfcePanelPlugin *plugin, GtkOrientation orientation, Fs
     gtk_progress_bar_set_orientation (GTK_PROGRESS_BAR(fsguard->progress_bar),
                                       orientation == GTK_ORIENTATION_HORIZONTAL ?
                                       GTK_PROGRESS_BOTTOM_TO_TOP : GTK_PROGRESS_LEFT_TO_RIGHT);
+    fsguard_set_size (plugin, xfce_panel_plugin_get_size (plugin), fsguard);
 }
-
-static gboolean
-fsguard_set_size (XfcePanelPlugin *plugin, int size, FsGuard *fsguard)
-{
-    DBG ("Set size to `%d'", size);
-
-    gtk_widget_set_size_request (fsguard->btn_panel, size, size);
-
-    GtkOrientation orientation = xfce_panel_plugin_get_orientation (plugin);
-    if (orientation == GTK_ORIENTATION_HORIZONTAL) {
-        gtk_widget_set_size_request (GTK_WIDGET(fsguard->progress_bar), BORDER, -1);
-        gtk_widget_set_size_request (GTK_WIDGET(plugin), -1, size);
-    } else {
-        gtk_widget_set_size_request (GTK_WIDGET(fsguard->progress_bar), -1, BORDER);
-        gtk_widget_set_size_request (GTK_WIDGET(plugin), size, -1);
-    }
-
-    fsguard_refresh_icon (fsguard);
-
-    return TRUE;
-}
+#endif
 
 static void
 fsguard_entry1_changed (GtkWidget *widget, FsGuard *fsguard)
@@ -580,7 +643,7 @@ fsguard_create_options (XfcePanelPlugin *plugin, FsGuard *fsguard)
 
     /* Configuration frame */
     GtkWidget *table1 = gtk_table_new (2, 3, FALSE);
-    GtkWidget *frame1 = xfce_create_framebox_with_content (_("Configuration"), table1);
+    GtkWidget *frame1 = xfce_gtk_frame_box_new_with_content (_("Configuration"), table1);
     gtk_table_set_row_spacings (GTK_TABLE (table1), BORDER);
     gtk_table_set_col_spacings (GTK_TABLE (table1), BORDER);
     gtk_container_set_border_width (GTK_CONTAINER (frame1), BORDER);
@@ -618,7 +681,7 @@ fsguard_create_options (XfcePanelPlugin *plugin, FsGuard *fsguard)
 
     /* Display frame */
     GtkWidget *table2 = gtk_table_new (2, 4, FALSE);
-    GtkWidget *frame2 = xfce_create_framebox_with_content (_("User Interface"), table2);
+    GtkWidget *frame2 = xfce_gtk_frame_box_new_with_content (_("User Interface"), table2);
     gtk_table_set_row_spacings (GTK_TABLE (table2), BORDER);
     gtk_table_set_col_spacings (GTK_TABLE (table2), BORDER);
     gtk_container_set_border_width (GTK_CONTAINER (frame2), BORDER);
@@ -710,6 +773,7 @@ fsguard_construct (XfcePanelPlugin *plugin)
       g_timeout_add (8192, (GSourceFunc) fsguard_check_fs, fsguard);
 
     gtk_container_add (GTK_CONTAINER (plugin), fsguard->ebox);
+    fsguard_set_size(fsguard->plugin, xfce_panel_plugin_get_size(fsguard->plugin), fsguard);
 
     g_signal_connect (plugin,
                       "free-data",
@@ -723,10 +787,18 @@ fsguard_construct (XfcePanelPlugin *plugin)
                       "size-changed",
                       G_CALLBACK (fsguard_set_size),
                       fsguard);
+#ifdef HAS_PANEL_49
+    g_signal_connect (plugin,
+                      "mode-changed",
+                      G_CALLBACK (fsguard_set_mode),
+                      fsguard);
+    xfce_panel_plugin_set_small (plugin, TRUE);
+#else
     g_signal_connect (plugin,
                       "orientation-changed",
                       G_CALLBACK (fsguard_set_orientation),
                       fsguard);
+#endif
     g_signal_connect (plugin,
                       "configure-plugin",
                       G_CALLBACK (fsguard_create_options),
@@ -735,7 +807,7 @@ fsguard_construct (XfcePanelPlugin *plugin)
     xfce_panel_plugin_menu_show_configure (plugin);
 }
 
-XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL (fsguard_construct);
+XFCE_PANEL_PLUGIN_REGISTER (fsguard_construct);
 
 // }}}
 
